from nwon_baseline.typings import BoundingBoxCoordinates as BoundingBoxCoordinates, BoundingBoxTuple as BoundingBoxTuple

def bounding_box_coordinates_from_tuple(box: BoundingBoxTuple): ...
def bounding_box_tuple_from_coordinates(box: BoundingBoxCoordinates) -> BoundingBoxTuple: ...



from nwon_baseline.print_helper.colored_printing import print_blue as print_blue, print_color as print_color, print_cyan as print_cyan, print_error as print_error, print_green as print_green, print_warning as print_warning
from nwon_baseline.print_helper.print_with_style import print_with_color_and_style as print_with_color_and_style, print_with_style as print_with_style



from nwon_baseline.pydantic.pydantic_model_to_dict import pydantic_model_to_dict as pydantic_model_to_dict
from nwon_baseline.pydantic.save_pydantic_model_instance import save_pydantic_model_instance_as_json as save_pydantic_model_instance_as_json
from nwon_baseline.pydantic.save_pydantic_model_schema import save_pydantic_model_schema as save_pydantic_model_schema



from nwon_baseline.type_helper.enum_type_guard import enum_type_guard as enum_type_guard
from nwon_baseline.type_helper.enum_value_list import enum_value_list as enum_value_list, enum_value_list_as_string as enum_value_list_as_string



from nwon_baseline.date_helper.country import timezone_offset_from_country_code as timezone_offset_from_country_code, timezones_from_country_code as timezones_from_country_code
from nwon_baseline.date_helper.datetime import datetime_now as datetime_now, datetime_with_timezone as datetime_with_timezone
from nwon_baseline.date_helper.time_differences import time_until as time_until



from nwon_baseline.directory_helper.directory_content import directory_names_in_directory as directory_names_in_directory, directory_paths_in_directory as directory_paths_in_directory, file_names_in_directory as file_names_in_directory, file_paths_in_directory as file_paths_in_directory, get_latest_file_in_directory as get_latest_file_in_directory
from nwon_baseline.directory_helper.directory_interactions import clean_directory as clean_directory, copy_directory as copy_directory, create_paths as create_paths



from nwon_baseline.typings.any_dict import AnyDict as AnyDict
from nwon_baseline.typings.bounding_box import BoundingBoxCoordinates as BoundingBoxCoordinates, BoundingBoxTuple as BoundingBoxTuple
from nwon_baseline.typings.celery_return import CeleryReturn as CeleryReturn
from nwon_baseline.typings.request_body_format import RequestBodyFormat as RequestBodyFormat
from nwon_baseline.typings.terminal_colors import TerminalColors as TerminalColors, TerminalStyling as TerminalStyling
from nwon_baseline.typings.version_change import VersionChange as VersionChange



from collections import defaultdict
from typing import List, Union

from django.db.models.signals import (
    ModelSignal,
    post_delete,
    post_init,
    post_migrate,
    post_save,
    pre_delete,
    pre_init,
    pre_migrate,
    pre_save,
)
from django.dispatch import Signal


class DisableSignals:
    """
    Class for managing the disabling of signals.
    """

    disabled_signals: List[Union[ModelSignal, Signal]]

    def __init__(self, disabled_signals: List[Union[ModelSignal, Signal]]):
        self.stashed_signals = defaultdict(list)
        self.disabled_signals = disabled_signals or [
            pre_init,
            post_init,
            pre_save,
            post_save,
            pre_delete,
            post_delete,
            pre_migrate,
            post_migrate,
        ]

    def __enter__(self):
        for signal in self.disabled_signals:
            self.disconnect(signal)

    def __exit__(self, exc_type, exc_val, exc_tb):
        for signal in list(self.stashed_signals):
            self.reconnect(signal)

    def disconnect(self, signal: Union[ModelSignal, Signal]):
        self.stashed_signals[signal] = signal.receivers
        signal.receivers = []

    def reconnect(self, signal: Union[ModelSignal, Signal]):
        signal.receivers = self.stashed_signals.get(signal, [])

        if signal in self.stashed_signals:
            del self.stashed_signals[signal]



from typing import BinaryIO, Optional, TextIO

def file_extension_from_url(image_url: str) -> str: ...
def file_extension_from_path(path: str) -> str: ...
def get_file_object(file_path: str) -> Optional[TextIO]: ...
def get_file_as_binary(file_path: str) -> Optional[BinaryIO]: ...
def read_file_as_binary(file_path: str) -> Optional[bytes]: ...
def read_file_content(file_path: str) -> Optional[str]: ...
def handle_uploaded_file(file, destination_path) -> str: ...



import threading
import types
from collections.abc import Callable, Iterable, Iterator
from pathlib import Path
from typing import Any

from django.apps.registry import Apps
from django.dispatch import Signal
from django.utils._os import _PathCompatible
from typing_extensions import ParamSpec

_P = ParamSpec("_P")

autoreload_started: Signal
file_changed: Signal
DJANGO_AUTORELOAD_ENV: str

def is_django_module(module: types.ModuleType) -> bool: ...
def is_django_path(path: _PathCompatible) -> bool: ...
def check_errors(fn: Callable[_P, Any]) -> Callable[_P, None]: ...
def raise_last_exception() -> None: ...
def ensure_echo_on() -> None: ...
def iter_all_python_module_files() -> set[Path]: ...
def iter_modules_and_files(
    modules: Iterable[types.ModuleType], extra_files: Iterable[_PathCompatible]
) -> frozenset[Path]: ...
def common_roots(paths: Iterable[Path]) -> Iterable[Path]: ...
def sys_path_directories() -> Iterator[Path]: ...
def get_child_arguments() -> list[str]: ...
def trigger_reload(filename: str) -> None: ...
def restart_with_reloader() -> int: ...

class BaseReloader:
    extra_files: set[Path]
    directory_globs: dict[Path, set[str]]
    def __init__(self) -> None: ...
    def watch_dir(self, path: _PathCompatible, glob: str) -> None: ...
    def watched_files(self, include_globs: bool = ...) -> Iterator[Path]: ...
    def wait_for_apps_ready(self, app_reg: Apps, django_main_thread: threading.Thread) -> bool: ...
    def run(self, django_main_thread: threading.Thread) -> None: ...
    def run_loop(self) -> None: ...
    def tick(self) -> Iterator[None]: ...
    @classmethod
    def check_availability(cls) -> bool | None: ...
    def notify_file_changed(self, path: _PathCompatible) -> None: ...
    @property
    def should_stop(self) -> bool: ...
    def stop(self) -> None: ...

class StatReloader(BaseReloader):
    SLEEP_TIME: int
    def snapshot_files(self) -> Iterator[tuple[Path, float]]: ...
    @classmethod
    def check_availability(cls) -> bool: ...

class WatchmanUnavailable(RuntimeError): ...

class WatchmanReloader(BaseReloader):
    processed_request: threading.Event
    client_timeout: int
    def __init__(self) -> None: ...
    @property
    def client(self) -> Any: ...
    def watched_roots(self, watched_files: Iterable[Path]) -> frozenset[Path]: ...
    def update_watches(self) -> None: ...
    def request_processed(self, **kwargs: Any) -> None: ...
    def check_server_status(self, inner_ex: BaseException | None = ...) -> bool: ...
    @classmethod
    def check_availability(cls) -> None: ...
    def stop(self) -> None: ...

def get_reloader() -> BaseReloader: ...
def start_django(reloader: BaseReloader, main_func: Callable, *args: Any, **kwargs: Any) -> None: ...
def run_with_reloader(main_func: Callable, *args: Any, **kwargs: Any) -> None: ...
